"""This package contains the resolvers for the Polywrap client."""
from .abc import *
from .aggregator import *
from .cache import *
from .extensions import *
from .legacy import *
from .package import *
from .recursive import *
from .redirect import *
from .static import *
from .wrapper import *
